// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "akonadi_serializer_mail_debug.h"


Q_LOGGING_CATEGORY(AKONADI_SERIALIZER_MAIL_LOG, "org.kde.pim.akonadi_serializer_mail", QtInfoMsg)

